@@ -19,6 +19,7 @@
 	   (added support for extended attributes.)
  */
 #define _XOPEN_SOURCE 500
+#define _BSD_SOURCE 1
 #include <fuse.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -31,7 +32,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <utime.h>
+
+#ifndef WITHOUT_XATTR
 #include <attr/xattr.h>
+#endif
 
 #include "parse_options.h"
 #include "tools.h"
@@ -56,19 +60,31 @@ static int mhdd_stat(const char *file_name, struct stat *buf)
 //statvfs
 static int mhdd_statfs(const char *path, struct statvfs *buf)
 {
-	int i;
+	int i, j;
 	struct statvfs * stats;
+	struct stat st;
+	dev_t * devices;
 
 	mhdd_debug(MHDD_MSG, "mhdd_statfs: %s\n", path);
 
 	stats = calloc(mhdd.cdirs, sizeof(struct statvfs));
+	devices = calloc(mhdd.cdirs, sizeof(dev_t));
 
 	for (i = 0; i < mhdd.cdirs; i++) {
 		int ret = statvfs(mhdd.dirs[i], stats+i);
 		if (ret != 0) {
 			free(stats);
+			free(devices);
 			return -errno;
 		}
+
+		ret = stat(mhdd.dirs[i], &st);
+		if (ret != 0) {
+			free(stats);
+			free(devices);
+			return -errno;
+		}
+		devices[i] = st.st_dev;
 	}
 
 	unsigned long
@@ -100,6 +116,21 @@ static int mhdd_statfs(const char *path, struct statvfs *buf)
 	memcpy(buf, stats, sizeof(struct statvfs));
 
 	for (i = 1; i<mhdd.cdirs; i++) {
+
+		/* if the device already processed, skip it */
+		if (devices[i]) {
+			int dup_found = 0;
+			for (j = 0; j < i; j++) {
+				if (devices[j] == devices[i]) {
+					dup_found = 1;
+					break;
+				}
+			}
+
+			if (dup_found)
+				continue;
+		}
+
 		if (buf->f_namemax<stats[i].f_namemax) {
 			buf->f_namemax = stats[i].f_namemax;
 		}
@@ -110,8 +141,9 @@ static int mhdd_statfs(const char *path, struct statvfs *buf)
 		buf->f_bfree  +=  stats[i].f_bfree;
 		buf->f_blocks +=  stats[i].f_blocks;
 	}
-	free(stats);
 
+	free(stats);
+	free(devices);
 	return 0;
 }
 
@@ -672,7 +704,7 @@ static int mhdd_utimens(const char *path, const struct timespec ts[2])
 		tv[1].tv_sec = ts[1].tv_sec;
 		tv[1].tv_usec = ts[1].tv_nsec / 1000;
 
-		res = utimes(object, tv);
+		res = lutimes(object, tv);
 		free(object);
 		if (res == -1)
 			return -errno;
@@ -833,7 +865,7 @@ static int mhdd_mknod(const char *path, mode_t mode, dev_t rdev)
 	return -errno;
 }
 
-#if _POSIX_SYNCHRONIZED_IO + 0 > 0
+#if _POSIX_SYNCHRONIZED_IO + 0 > 0 || defined(__FreeBSD__)
 #undef HAVE_FDATASYNC
 #else
 #define HAVE_FDATASYNC 1
@@ -869,6 +901,8 @@ static int mhdd_fsync(const char *path, int isdatasync,
 }
 
 // Define extended attribute support
+
+#ifndef WITHOUT_XATTR
 static int mhdd_setxattr(const char *path, const char *attrname,
                 const char *attrval, size_t attrvalsize, int flags)
 {
@@ -884,7 +918,9 @@ static int mhdd_setxattr(const char *path, const char *attrname,
         if (res == -1) return -errno;
         return 0;
 }
+#endif
 
+#ifndef WITHOUT_XATTR
 static int mhdd_getxattr(const char *path, const char *attrname, char *buf, size_t count)
 {
         int size = 0;
@@ -900,7 +936,9 @@ static int mhdd_getxattr(const char *path, const char *attrname, char *buf, size
         if (size == -1) return -errno;
         return size;
 }
+#endif
 
+#ifndef WITHOUT_XATTR
 static int mhdd_listxattr(const char *path, char *buf, size_t count)
 {
         int ret = 0;
@@ -917,7 +955,9 @@ static int mhdd_listxattr(const char *path, char *buf, size_t count)
         if (ret == -1) return -errno;
         return ret;
 }
+#endif
 
+#ifndef WITHOUT_XATTR
 static int mhdd_removexattr(const char *path, const char *attrname)
 {
 	char * real_path = find_path(path);
@@ -933,35 +973,38 @@ static int mhdd_removexattr(const char *path, const char *attrname)
         if (res == -1) return -errno;
         return 0;
 }
+#endif
 
 // functions links
 static struct fuse_operations mhdd_oper = {
-	.getattr    = mhdd_stat,
-	.statfs     = mhdd_statfs,
-	.readdir    = mhdd_readdir,
-	.readlink   = mhdd_readlink,
-	.open       = mhdd_fileopen,
-	.release    = mhdd_release,
-	.read       = mhdd_read,
-	.write      = mhdd_write,
-	.create     = mhdd_create,
-	.truncate   = mhdd_truncate,
-	.ftruncate  = mhdd_ftruncate,
-	.access     = mhdd_access,
-	.mkdir      = mhdd_mkdir,
-	.rmdir      = mhdd_rmdir,
-	.unlink     = mhdd_unlink,
-	.rename     = mhdd_rename,
-	.utimens    = mhdd_utimens,
-	.chmod      = mhdd_chmod,
-	.chown      = mhdd_chown,
-	.symlink    = mhdd_symlink,
-	.mknod      = mhdd_mknod,
-	.fsync      = mhdd_fsync,
-        .setxattr   = mhdd_setxattr,
-        .getxattr   = mhdd_getxattr,
-        .listxattr  = mhdd_listxattr,
-        .removexattr= mhdd_removexattr,
+	.getattr    	= mhdd_stat,
+	.statfs     	= mhdd_statfs,
+	.readdir    	= mhdd_readdir,
+	.readlink   	= mhdd_readlink,
+	.open       	= mhdd_fileopen,
+	.release    	= mhdd_release,
+	.read       	= mhdd_read,
+	.write      	= mhdd_write,
+	.create     	= mhdd_create,
+	.truncate   	= mhdd_truncate,
+	.ftruncate  	= mhdd_ftruncate,
+	.access     	= mhdd_access,
+	.mkdir      	= mhdd_mkdir,
+	.rmdir      	= mhdd_rmdir,
+	.unlink     	= mhdd_unlink,
+	.rename     	= mhdd_rename,
+	.utimens    	= mhdd_utimens,
+	.chmod      	= mhdd_chmod,
+	.chown      	= mhdd_chown,
+	.symlink    	= mhdd_symlink,
+	.mknod      	= mhdd_mknod,
+	.fsync      	= mhdd_fsync,
+#ifndef WITHOUT_XATTR
+        .setxattr   	= mhdd_setxattr,
+        .getxattr   	= mhdd_getxattr,
+        .listxattr  	= mhdd_listxattr,
+        .removexattr	= mhdd_removexattr,
+#endif
 };
 
 
